// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

using System;
using Xunit;

namespace System.Tests
{
    public static class DuplicateWaitObjectExceptionTests
    {
        private const int COR_E_DUPLICATEWAITOBJECT = unchecked((int)0x80131529);

        [Fact]
        public static void Ctor_Empty()
        {
            var exception = new DuplicateWaitObjectException();
            Assert.NotEmpty(exception.Message);
            Assert.Equal(COR_E_DUPLICATEWAITOBJECT, exception.HResult);
        }

        [Fact]
        public static void Ctor_String()
        {
            string parameterName = "THISISAPARAMETERNAME";
            var exception = new DuplicateWaitObjectException(parameterName);
            Assert.Contains(parameterName, exception.Message);
            Assert.Equal(COR_E_DUPLICATEWAITOBJECT, exception.HResult);
        }

        [Fact]
        public static void Ctor_String_Exception()
        {
            string message = "CreatedDuplicateWaitObjectException";
            var innerException = new Exception("Created inner exception");
            var exception = new DuplicateWaitObjectException(message, innerException);
            Assert.Equal(message, exception.Message);
            Assert.Equal(COR_E_DUPLICATEWAITOBJECT, exception.HResult);
            Assert.Same(innerException, exception.InnerException);
            Assert.Equal(innerException.HResult, exception.InnerException.HResult);
        }

        [Fact]
        public static void Ctor_String_String()
        {
            string parameterName = "THISISAPARAMETERNAME";
            string message = "CreatedDuplicateWaitObjectException";
            var exception = new DuplicateWaitObjectException(parameterName, message);
            Assert.Contains(parameterName, exception.Message);
            Assert.Contains(message, exception.Message);
        }
    }
}
